/* -*- c++ -*- */
/*
 * Copyright 2004 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef INCLUDED_USRP0_SOURCE_C_H
#define INCLUDED_USRP0_SOURCE_C_H

#include <gr_sync_block.h>
#include <stdexcept>

class usrp0_rx;


class usrp0_source_c;
typedef boost::shared_ptr<usrp0_source_c> usrp0_source_c_sptr;


// public shared_ptr constructor

usrp0_source_c_sptr
usrp0_make_source_c (int which_board, unsigned int decim_rate) throw (std::runtime_error);


/*!
 * \brief interface to Universal Software Radio Peripheral Rx path
 * \ingroup source
 */
class usrp0_source_c : public gr_sync_block {
 private:
  usrp0_rx		*d_usrp;
  int			 d_noverruns;
  
  friend usrp0_source_c_sptr
  usrp0_make_source_c (int which_board, unsigned int decim_rate) throw (std::runtime_error);

 protected:
  usrp0_source_c (int which_board, unsigned int decim_rate) throw (std::runtime_error);

 public:
  ~usrp0_source_c ();

  int work (int noutput_items,
	    gr_vector_const_void_star &input_items,
	    gr_vector_void_star &output_items);

  /*!
   * \brief set ADC clock divisor.  Must be in the range [2,6]
   *
   * Set the clock divisor used to generate the ADC clock from the
   * master oscillator.  \p div must be in the range [2,6].
   * The default value is 2, giving a sample rate of 62.5 MS/s
   */
  bool set_adc_clk_div (unsigned int div);
  
  /*!
   * \brief Set decimator rate.  \p rate must be in [2, 256].
   *
   * The final complex sample rate across the USB is
   *   adc_freq () / decim_rate ()
   */
  bool set_decim_rate (unsigned int rate);

  /*!
   * \brief set the center frequency of the digital down converter.
   *
   * \p channel must be 0.  \p freq is the center frequency in Hz.
   * It must be in the range [-FIXME, FIXME].  The frequency specified is
   * quantized.  Use rx_freq to retrieve the actual value used.
   */
  bool set_rx_freq (int channel, double freq);

  // ACCESSORS

  unsigned int adc_clk_div () const;
  long adc_freq () const;
  unsigned int decim_rate () const;
  double rx_freq (int channel) const;
  int noverruns () const { return d_noverruns; }
};

#endif /* INCLUDED_USRP0_SOURCE_C_H */
